#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>

#include <mysql.h>
#include <NdbApi.hpp>

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 1;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, pargc, pargv);

	// parse command line arguments
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
		{0, 0, 0, 0}
	};
	char *short_options = "c:";
	int opt, option_index;

	opterr = 0;
	do {
		opt = getopt_long(*pargc, *pargv, short_options, long_options, &option_index);
		switch (opt) {
		case 'c':
			connectstr = optarg;
			break;
		case 't':
			no_retries = atoi(optarg);
		default:
			break;
		}
	} while ( opt != -1);

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
	if (conn->wait_until_ready(30,0) < 0)
	{
		fprintf(stderr, "Cluster was not ready within 30 secs.\n");
		exit(EXIT_FAILURE);
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}



int main(int argc, char **argv)
{
    Ndb_cluster_connection *conn;
	char constr[1024];
	char c32[33];

	// connect
	conn = example_init(&argc, &argv);

	// create Ndb object
	Ndb *ndb = new Ndb(conn, "test");
	if (!ndb) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}
	if (ndb->init()) {
	  fprintf(stderr, "Can't create NDB object\n");
	  exit(3);
	}

	// get dictionary
    const NdbDictionary::Dictionary* dict= ndb->getDictionary();
	if (!dict) {
	  fprintf(stderr, "Can't create dict object\n");
	  exit(3);
	}
	
	// lookup session table
    const NdbDictionary::Table *table= dict->getTable("session");
	if (!table) {
	  fprintf(stderr, "Can't create table object\n");
	  exit(3);
	}
	
	// start transaction
    NdbTransaction   *trans = ndb->startTransaction();
	if (!trans) {
	  fprintf(stderr, "Can't create trans object\n");
	  exit(3);
	}

	// get operation
    NdbOperation *op = trans->getNdbOperation(table);
	if (!op) {
	  fprintf(stderr, "Can't create operation object\n");
	  exit(3);
	}

	// read a row ...
	if (op->readTuple(NdbOperation::LM_Exclusive) != 0) {
	  fprintf(stderr, "Can't readTuple\n");
	  exit(3);
	}
	// ... by primary key
    if (0 != op->equal("id", "aa8c6091ff4915797545b81455680bf7")) {
	  fprintf(stderr, "Can't equal\n");
	  exit(3);
	}

	// get blob handle 
	NdbBlob *blob = op->getBlobHandle("data");
	if (NULL == blob) {
	  fprintf(stderr, "Can't create blob object\n");
	  exit(3);
	}

	// execute transacion
	// CAUTION: the NoCommit is crutial, else the blob won't be active
	if (trans->execute( NdbTransaction::NoCommit ) == -1) {
	  fprintf(stderr, "Can't execute transaction\n");
	  NdbError err = trans->getNdbError();								
	  fprintf(stderr, "%s\n", err.message); 
	  exit(3);
	}

	// get the first up to 1000 bytes from the blob
	Uint64 len;
	Uint32 actual = 1000;

	char *d = (char *)malloc(1001);
	blob->getLength(len);

	blob->readData((void *)d, actual);
	d[actual] = '\0';

	printf("%d of %d chars: '%s'\n", (int)actual, (int)len, d);

	// now try to truncate it
	len = 100;
	int stat = blob->truncate(len);
	 
	printf("stat: %d\n", stat);

	// commit
	if (trans->execute( NdbTransaction::Commit ) == -1) {
	  fprintf(stderr, "Can't commit transaction\n");
	  NdbError err = trans->getNdbError();								
	  fprintf(stderr, "%s\n", err.message); 
	  exit(3);
	}

	ndb->closeTransaction(trans);

	example_end(conn);

	return EXIT_SUCCESS;
}
